@@ -16,6 +16,7 @@
  * You should have received a copy of the GNU General Public License along
  * with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
+#include "config.h"
 
 #include "gcal-edit-dialog.h"
 #include "gcal-time-selector.h"
@@ -33,7 +34,11 @@ struct _GcalEditDialog
 
   GcalManager      *manager;
 
+  /* titlebar */
   GtkWidget        *titlebar;
+  GtkWidget        *title_label;
+  GtkWidget        *subtitle_label;
+
   GtkWidget        *lock;
   GtkWidget        *source_image;
   GtkWidget        *source_label;
@@ -54,13 +59,25 @@ struct _GcalEditDialog
   GtkWidget        *location_entry;
   GtkWidget        *notes_text;
 
+  GtkWidget        *alarms_listbox;
+
+  /* Add Alarms popover buttons */
+  GtkWidget        *five_minutes_button;
+  GtkWidget        *ten_minutes_button;
+  GtkWidget        *thirty_minutes_button;
+  GtkWidget        *one_hour_button;
+  GtkWidget        *one_day_button;
+  GtkWidget        *two_days_button;
+  GtkWidget        *three_days_button;
+  GtkWidget        *one_week_button;
+
   /* actions */
   GMenu              *sources_menu;
-  GSimpleAction      *action;
   GSimpleActionGroup *action_group;
 
   /* new data holders */
   GcalEvent        *event;
+  ESource          *selected_source;
 
   /* flags */
   gboolean          format_24h;
@@ -70,7 +87,7 @@ struct _GcalEditDialog
 
 static void        fill_sources_menu                      (GcalEditDialog    *dialog);
 
-static void        on_calendar_selected                   (GtkWidget         *menu_item,
+static void        on_calendar_selected                   (GSimpleAction     *menu_item,
                                                            GVariant          *value,
                                                            gpointer           user_data);
 
@@ -97,6 +114,9 @@ static void        gcal_edit_dialog_action_button_clicked (GtkWidget         *wi
 static void        gcal_edit_dialog_all_day_changed       (GtkWidget         *widget,
                                                            gpointer           user_data);
 
+static void        add_alarm_button_clicked               (GtkWidget         *button,
+                                                           GcalEditDialog    *self);
+
 G_DEFINE_TYPE (GcalEditDialog, gcal_edit_dialog, GTK_TYPE_DIALOG)
 
 enum
@@ -108,6 +128,11 @@ enum
   LAST_PROP
 };
 
+const GActionEntry action_entries[] =
+{
+  { "select-calendar", on_calendar_selected, "s" },
+};
+
 static void
 fill_sources_menu (GcalEditDialog *dialog)
 {
@@ -131,20 +156,20 @@ fill_sources_menu (GcalEditDialog *dialog)
 
       /* retrieve color */
       get_color_name_from_source (source, &color);
-      pix = gcal_get_pixbuf_from_color (&color, 16);;
+      pix = get_circle_pixbuf_from_color (&color, 16);;
 
       /* menu item */
-      item = g_menu_item_new (e_source_get_display_name (source), "select_calendar");
+      item = g_menu_item_new (e_source_get_display_name (source), "select-calendar");
       g_menu_item_set_icon (item, G_ICON (pix));
 
       /* set insensitive for read-only calendars */
       if (!gcal_manager_is_client_writable (dialog->manager, source))
         {
-          g_menu_item_set_action_and_target_value (item, "select_calendar", NULL);
+          g_menu_item_set_action_and_target_value (item, "select-calendar", NULL);
         }
       else
         {
-          g_menu_item_set_action_and_target_value (item, "select_calendar",
+          g_menu_item_set_action_and_target_value (item, "select-calendar",
                                                    g_variant_new_string (e_source_get_uid (source)));
         }
 
@@ -163,17 +188,17 @@ fill_sources_menu (GcalEditDialog *dialog)
 }
 
 static void
-on_calendar_selected (GtkWidget *menu_item,
-                      GVariant  *value,
-                      gpointer   user_data)
+on_calendar_selected (GSimpleAction *action,
+                      GVariant      *value,
+                      gpointer       user_data)
 {
-  GcalEditDialog *dialog;
+  GcalEditDialog *self;
   GList *list;
   GList *aux;
   gchar *uid;
 
-  dialog = GCAL_EDIT_DIALOG (user_data);
-  list = gcal_manager_get_sources (dialog->manager);
+  self = GCAL_EDIT_DIALOG (user_data);
+  list = gcal_manager_get_sources (self->manager);
 
   /* retrieve selected calendar uid */
   g_variant_get (value, "s", &uid);
@@ -192,12 +217,13 @@ on_calendar_selected (GtkWidget *menu_item,
         /* retrieve color */
         get_color_name_from_source (source, &color);
 
-        pix = gcal_get_pixbuf_from_color (&color, 16);
-        gtk_image_set_from_pixbuf (GTK_IMAGE (dialog->source_image), pix);
+        pix = get_circle_pixbuf_from_color (&color, 16);
+        gtk_image_set_from_pixbuf (GTK_IMAGE (self->source_image), pix);
         g_object_unref (pix);
 
-        gcal_event_set_source (dialog->event, source);
-        gtk_header_bar_set_subtitle (GTK_HEADER_BAR (dialog->titlebar),
+        self->selected_source = source;
+
+        gtk_header_bar_set_subtitle (GTK_HEADER_BAR (self->titlebar),
                                      e_source_get_display_name (source));
         break;
       }
@@ -231,6 +257,43 @@ update_summary (GtkEntry   *entry,
   gtk_widget_set_sensitive (dialog->done_button, gtk_entry_get_text_length (entry) > 0);
 }
 
+static gint
+sort_alarms_func (GtkListBoxRow *a,
+                  GtkListBoxRow *b,
+                  gpointer       user_data)
+{
+  ECalComponentAlarm *alarm_a, *alarm_b;
+  GcalEvent *event_a, *event_b;
+  gint minutes_a, minutes_b;
+
+  alarm_a = g_object_get_data (G_OBJECT (a), "alarm");
+  alarm_b = g_object_get_data (G_OBJECT (b), "alarm");
+  event_a = g_object_get_data (G_OBJECT (a), "event");
+  event_b = g_object_get_data (G_OBJECT (b), "event");
+
+  minutes_a = get_alarm_trigger_minutes (event_a, alarm_a);
+  minutes_b = get_alarm_trigger_minutes (event_b, alarm_b);
+
+  return minutes_a - minutes_b;
+}
+
+static void
+fix_reminders_label_height_cb (GtkWidget    *summary_label,
+                               GdkRectangle *allocation,
+                               GtkWidget    *reminders_label)
+{
+  gtk_widget_set_size_request (reminders_label, -1, allocation->height);
+}
+
+static void
+update_revealer_visibility_cb (GtkRevealer *revealer)
+{
+  if (gtk_revealer_get_reveal_child (revealer))
+    gtk_widget_set_visible (GTK_WIDGET (revealer), TRUE);
+  else if (!gtk_revealer_get_child_revealed (revealer))
+    gtk_widget_set_visible (GTK_WIDGET (revealer), FALSE);
+}
+
 static void
 gcal_edit_dialog_get_property (GObject    *object,
                                guint       prop_id,
@@ -339,6 +402,15 @@ gcal_edit_dialog_class_init (GcalEditDialogClass *klass)
                                                          TRUE,
                                                          G_PARAM_READWRITE));
 
+  /* Alarms */
+  gtk_widget_class_bind_template_child (widget_class, GcalEditDialog, five_minutes_button);
+  gtk_widget_class_bind_template_child (widget_class, GcalEditDialog, ten_minutes_button);
+  gtk_widget_class_bind_template_child (widget_class, GcalEditDialog, thirty_minutes_button);
+  gtk_widget_class_bind_template_child (widget_class, GcalEditDialog, one_hour_button);
+  gtk_widget_class_bind_template_child (widget_class, GcalEditDialog, one_day_button);
+  gtk_widget_class_bind_template_child (widget_class, GcalEditDialog, two_days_button);
+  gtk_widget_class_bind_template_child (widget_class, GcalEditDialog, three_days_button);
+  gtk_widget_class_bind_template_child (widget_class, GcalEditDialog, one_week_button);
   /* Buttons */
   gtk_widget_class_bind_template_child (widget_class, GcalEditDialog, done_button);
   gtk_widget_class_bind_template_child (widget_class, GcalEditDialog, cancel_button);
@@ -352,35 +424,45 @@ gcal_edit_dialog_class_init (GcalEditDialogClass *klass)
   gtk_widget_class_bind_template_child (widget_class, GcalEditDialog, end_date_selector);
   gtk_widget_class_bind_template_child (widget_class, GcalEditDialog, location_entry);
   /* Other */
+  gtk_widget_class_bind_template_child (widget_class, GcalEditDialog, alarms_listbox);
   gtk_widget_class_bind_template_child (widget_class, GcalEditDialog, notes_text);
   gtk_widget_class_bind_template_child (widget_class, GcalEditDialog, all_day_check);
   gtk_widget_class_bind_template_child (widget_class, GcalEditDialog, titlebar);
+  gtk_widget_class_bind_template_child (widget_class, GcalEditDialog, title_label);
+  gtk_widget_class_bind_template_child (widget_class, GcalEditDialog, subtitle_label);
   gtk_widget_class_bind_template_child (widget_class, GcalEditDialog, lock);
   gtk_widget_class_bind_template_child (widget_class, GcalEditDialog, source_image);
   gtk_widget_class_bind_template_child (widget_class, GcalEditDialog, sources_popover);
 
   /* callbacks */
+  gtk_widget_class_bind_template_callback (widget_class, add_alarm_button_clicked);
+  gtk_widget_class_bind_template_callback (widget_class, fix_reminders_label_height_cb);
   gtk_widget_class_bind_template_callback (widget_class, gcal_edit_dialog_action_button_clicked);
   gtk_widget_class_bind_template_callback (widget_class, gcal_edit_dialog_all_day_changed);
   gtk_widget_class_bind_template_callback (widget_class, update_summary);
   gtk_widget_class_bind_template_callback (widget_class, update_location);
+  gtk_widget_class_bind_template_callback (widget_class, update_revealer_visibility_cb);
 }
 
 static void
 gcal_edit_dialog_init (GcalEditDialog *self)
 {
   self->writable = TRUE;
-  self->format_24h = TRUE;
 
   gtk_widget_init_template (GTK_WIDGET (self));
+
+  gtk_list_box_set_sort_func (GTK_LIST_BOX (self->alarms_listbox),
+                              sort_alarms_func,
+                              self,
+                              NULL);
 }
 
 static void
 gcal_edit_dialog_constructed (GObject* object)
 {
-  GcalEditDialog *dialog;
+  GcalEditDialog *self;
 
-  dialog = GCAL_EDIT_DIALOG (object);
+  self = GCAL_EDIT_DIALOG (object);
 
   /* chaining up */
   G_OBJECT_CLASS (gcal_edit_dialog_parent_class)->constructed (object);
@@ -388,20 +470,18 @@ gcal_edit_dialog_constructed (GObject* object)
   gtk_window_set_title (GTK_WINDOW (object), "");
 
   /* titlebar */
-  gtk_window_set_titlebar (GTK_WINDOW (object), dialog->titlebar);
+  gtk_window_set_titlebar (GTK_WINDOW (object), self->titlebar);
 
-  /* actions */
-  dialog->action_group = g_simple_action_group_new ();
-  gtk_widget_insert_action_group (GTK_WIDGET (object),
+  /* Actions */
+  self->action_group = g_simple_action_group_new ();
+  g_action_map_add_action_entries (G_ACTION_MAP (self->action_group),
+                                   action_entries,
+                                   G_N_ELEMENTS (action_entries),
+                                   self);
+
+  gtk_widget_insert_action_group (GTK_WIDGET (self),
                                   "edit",
-                                  G_ACTION_GROUP (dialog->action_group));
-
-  dialog->action = g_simple_action_new ("select_calendar", G_VARIANT_TYPE_STRING);
-  g_signal_connect (dialog->action,
-                    "activate",
-                    G_CALLBACK (on_calendar_selected), object);
-
-  g_action_map_add_action (G_ACTION_MAP (dialog->action_group), G_ACTION (dialog->action));
+                                  G_ACTION_GROUP (self->action_group));
 }
 
 static void
@@ -412,7 +492,6 @@ gcal_edit_dialog_finalize (GObject *object)
   dialog = GCAL_EDIT_DIALOG (object);
 
   g_clear_object (&dialog->action_group);
-  g_clear_object (&dialog->action);
   g_clear_object (&dialog->manager);
   g_clear_object (&dialog->event);
 
@@ -516,25 +595,35 @@ gcal_edit_dialog_action_button_clicked (GtkWidget *widget,
       start_date = gcal_edit_dialog_get_date_start (dialog);
       end_date = gcal_edit_dialog_get_date_end (dialog);
 
-      /*
-       * The end date for multi-day events is exclusive, so we bump it by a day.
-       * This fixes the discrepancy between the end day of the event and how it
-       * is displayed in the month view. See bug 769300.
-       */
-      if (all_day)
-        {
-          GDateTime *fake_end_date = g_date_time_add_days (end_date, 1);
-
-          g_clear_pointer (&end_date, g_date_time_unref);
-          end_date = fake_end_date;
-        }
-
       gcal_event_set_date_start (dialog->event, start_date);
       gcal_event_set_date_end (dialog->event, end_date);
 
       g_clear_pointer (&start_date, g_date_time_unref);
       g_clear_pointer (&end_date, g_date_time_unref);
 
+      /* Update the source if needed */
+      if (dialog->selected_source &&
+          gcal_event_get_source (dialog->event) != dialog->selected_source)
+        {
+          /*
+           * When we're creating the event, it has no source set, so we simply
+           * set the source. When editing the event, however, we're actually
+           * moving it from one source to another.
+           */
+          if (!gcal_event_get_source (dialog->event))
+            {
+              gcal_event_set_source (dialog->event, dialog->selected_source);
+            }
+          else
+            {
+              gcal_manager_move_event_to_source (dialog->manager,
+                                                 dialog->event,
+                                                 dialog->selected_source);
+            }
+        }
+
+      dialog->selected_source = NULL;
+
       /* Send the response */
       gtk_dialog_response (GTK_DIALOG (dialog),
                            dialog->event_is_new ? GCAL_RESPONSE_CREATE_EVENT : GCAL_RESPONSE_SAVE_EVENT);
@@ -557,6 +646,321 @@ gcal_edit_dialog_all_day_changed (GtkWidget *widget,
   gtk_widget_set_sensitive (dialog->end_time_selector, !active);
 }
 
+/*
+ * Alarm related functions
+ */
+static GtkWidget*
+get_row_fow_alarm_trigger_minutes (GcalEditDialog *self,
+                                   gint            minutes)
+{
+  gint i;
+
+  struct
+  {
+    gint       minutes;
+    GtkWidget *button;
+  } minutes_button[] = {
+      { 5,     self->five_minutes_button },
+      { 10,    self->ten_minutes_button },
+      { 30,    self->thirty_minutes_button },
+      { 60,    self->one_hour_button },
+      { 1440,  self->one_day_button },
+      { 2880,  self->two_days_button },
+      { 4320,  self->three_days_button },
+      { 10080, self->one_week_button }
+  };
+
+  for (i = 0; i < G_N_ELEMENTS (minutes_button); i++)
+    {
+      if (minutes_button[i].minutes == minutes)
+        return minutes_button[i].button;
+    }
+
+  return NULL;
+}
+static void
+remove_button_clicked (GtkButton *button,
+                       GtkWidget *row)
+{
+  ECalComponentAlarm *alarm;
+  GcalEditDialog *self;
+  GtkWidget *alarm_button;
+  GcalEvent *event;
+  gint trigger_minutes;
+
+  self = GCAL_EDIT_DIALOG (gtk_widget_get_toplevel (row));
+  alarm = g_object_get_data (G_OBJECT (row), "alarm");
+  event = g_object_get_data (G_OBJECT (row), "event");
+  trigger_minutes = get_alarm_trigger_minutes (event, alarm);
+
+  /*
+   * Make the button sensitive again
+   */
+  alarm_button = get_row_fow_alarm_trigger_minutes (self, trigger_minutes);
+
+  if (alarm_button)
+    gtk_widget_set_sensitive (alarm_button, TRUE);
+
+  gcal_event_remove_alarm (event, trigger_minutes);
+
+  gcal_manager_update_event (self->manager, event);
+
+  gtk_widget_destroy (row);
+
+  /*
+   * In order to not allocate a spacing between the listbox and the
+   * add alarms button, we should always keep the listbox:visible property
+   * updated.
+   */
+  gtk_widget_set_visible (self->alarms_listbox, gcal_event_has_alarms (self->event));
+}
+
+static void
+sound_toggle_changed (GtkToggleButton *button,
+                      GtkWidget       *row)
+{
+  ECalComponentAlarmAction action;
+  ECalComponentAlarm *alarm;
+  GtkWidget *image;
+  gboolean has_sound;
+
+  alarm = g_object_get_data (G_OBJECT (row), "alarm");
+  image = gtk_bin_get_child (GTK_BIN (button));
+  has_sound = gtk_toggle_button_get_active (button);
+
+  /* Setup the alarm action */
+  action = has_sound ? E_CAL_COMPONENT_ALARM_AUDIO : E_CAL_COMPONENT_ALARM_DISPLAY;
+
+  e_cal_component_alarm_set_action (alarm, action);
+
+  /* Update the volume icon */
+  gtk_image_set_from_icon_name (GTK_IMAGE (image),
+                                has_sound ? "audio-volume-high-symbolic" : "audio-volume-muted-symbolic",
+                                GTK_ICON_SIZE_BUTTON);
+
+}
+
+static GtkWidget*
+create_row_for_alarm (GcalEvent          *event,
+                      ECalComponentAlarm *alarm)
+{
+  ECalComponentAlarmAction action;
+  GtkBuilder *builder;
+  GtkWidget *label, *main_box, *row, *remove_button;
+  GtkWidget *volume_button, *volume_icon;
+  gboolean has_sound;
+  gchar *text;
+  gint trigger_minutes;
+
+  trigger_minutes = get_alarm_trigger_minutes (event, alarm);
+
+  /* Something bad happened */
+  if (trigger_minutes < 0)
+    return NULL;
+
+  if (trigger_minutes < 60)
+    {
+      text = g_strdup_printf (g_dngettext (GETTEXT_PACKAGE,
+                                           "%d minute before",
+                                           "%d minutes before",
+                                           trigger_minutes),
+                              trigger_minutes);
+    }
+  else if (trigger_minutes < 1440)
+    {
+      text = g_strdup_printf (g_dngettext (GETTEXT_PACKAGE,
+                                           "%d hour before",
+                                           "%d hours before",
+                                           trigger_minutes / 60),
+                              trigger_minutes / 60);
+    }
+  else if (trigger_minutes < 10080)
+    {
+      text = g_strdup_printf (g_dngettext (GETTEXT_PACKAGE,
+                                           "%d day before",
+                                           "%d days before",
+                                           trigger_minutes / 1440),
+                              trigger_minutes / 1440);
+    }
+  else
+    {
+      text = g_strdup_printf (g_dngettext (GETTEXT_PACKAGE,
+                                           "%d week before",
+                                           "%d weeks before",
+                                           trigger_minutes / 10080),
+                              trigger_minutes / 10080);
+    }
+
+  /* The row */
+  row = gtk_list_box_row_new ();
+
+  g_object_set_data (G_OBJECT (row), "alarm", alarm);
+  g_object_set_data (G_OBJECT (row), "event", event);
+
+  /* Build the UI */
+  builder = gtk_builder_new_from_resource ("/org/gnome/calendar/alarm-row.ui");
+
+#define WID(x) (GTK_WIDGET (gtk_builder_get_object (builder, x)))
+
+  label = WID ("label");
+  gtk_label_set_label (GTK_LABEL (label), text);
+
+  /* Retrieves the actions associated to the alarm */
+  e_cal_component_alarm_get_action (alarm, &action);
+
+  /* Updates the volume button to match the action */
+  has_sound = action == E_CAL_COMPONENT_ALARM_AUDIO;
+
+  volume_button = WID ("volume_button");
+  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (volume_button), has_sound);
+
+  volume_icon = WID ("volume_icon");
+  gtk_image_set_from_icon_name (GTK_IMAGE (volume_icon),
+                                has_sound ? "audio-volume-high-symbolic" : "audio-volume-muted-symbolic",
+                                GTK_ICON_SIZE_BUTTON);
+
+  g_signal_connect (volume_button,
+                    "toggled",
+                    G_CALLBACK (sound_toggle_changed),
+                    row);
+
+  /* Remove button */
+  remove_button = WID ("remove_button");
+
+  g_signal_connect (remove_button,
+                    "clicked",
+                    G_CALLBACK (remove_button_clicked),
+                    row);
+
+  main_box = WID ("main_box");
+  gtk_container_add (GTK_CONTAINER (row), main_box);
+
+  gtk_widget_show_all (row);
+
+  g_clear_object (&builder);
+  g_free (text);
+
+#undef WID
+
+  return row;
+}
+
+static void
+setup_alarms (GcalEditDialog *self)
+{
+  GList *alarms, *l;
+  gint i;
+
+  struct
+  {
+    gint       minutes;
+    GtkWidget *button;
+  } minutes_button[] = {
+      { 5,     self->five_minutes_button },
+      { 10,    self->ten_minutes_button },
+      { 30,    self->thirty_minutes_button },
+      { 60,    self->one_hour_button },
+      { 1440,  self->one_day_button },
+      { 2880,  self->two_days_button },
+      { 4320,  self->three_days_button },
+      { 10080, self->one_week_button }
+  };
+
+  gtk_widget_set_visible (self->alarms_listbox, gcal_event_has_alarms (self->event));
+
+  alarms = gcal_event_get_alarms (self->event);
+
+  /* Remove previous alarms */
+  gtk_container_foreach (GTK_CONTAINER (self->alarms_listbox),
+                         (GtkCallback) gtk_widget_destroy,
+                         NULL);
+
+  /*
+   * We start by making all alarm buttons sensitive,
+   * and only make them insensitive when needed.
+   */
+  for (i = 0; i < G_N_ELEMENTS (minutes_button); i++)
+    gtk_widget_set_sensitive (minutes_button[i].button, TRUE);
+
+  for (l = alarms; l != NULL; l = l->next)
+    {
+      GtkWidget *row;
+      gint minutes;
+
+      row = create_row_for_alarm (self->event, l->data);
+
+      if (!row)
+        continue;
+
+      /* Make already-added alarm buttons insensitive */
+      minutes = get_alarm_trigger_minutes (self->event, l->data);
+
+      for (i = 0; i < G_N_ELEMENTS (minutes_button); i++)
+        {
+          if (minutes_button[i].minutes == minutes)
+            gtk_widget_set_sensitive (minutes_button[i].button, FALSE);
+        }
+
+      /* Add the row */
+      gtk_container_add (GTK_CONTAINER (self->alarms_listbox), row);
+    }
+
+  g_list_free (alarms);
+}
+
+static void
+add_alarm_button_clicked (GtkWidget      *button,
+                          GcalEditDialog *self)
+{
+  gint i, minutes;
+
+  struct
+  {
+    GtkWidget *button;
+    gint       minutes;
+  } button_minutes[] = {
+      { self->five_minutes_button,   5 },
+      { self->ten_minutes_button,    10 },
+      { self->thirty_minutes_button, 30 },
+      { self->one_hour_button,       60 },
+      { self->one_day_button,        1440 },
+      { self->two_days_button,       2880 },
+      { self->three_days_button,     4320 },
+      { self->one_week_button,       10080 },
+      { NULL, 0 }
+  };
+
+  /* Search for the button minute */
+  minutes = 0;
+
+  for (i = 0; i < G_N_ELEMENTS (button_minutes); i++)
+    {
+      if (button_minutes[i].button == button)
+        {
+          minutes = button_minutes[i].minutes;
+          break;
+        }
+    }
+
+  if (minutes == 0)
+    return;
+
+  /* Add the alarm */
+  gcal_event_add_alarm (self->event, minutes, FALSE);
+
+  /*
+   * Instead of manually handling stuff, simply remove all alarms and
+   * add back again.
+   */
+  setup_alarms (self);
+
+  /*
+   * Since we don't allow more than 1 alarm per time, set the button
+   * to insensitive so it cannot be triggered anymore.
+   */
+  gtk_widget_set_sensitive (button, FALSE);
+}
+
 /* Public API */
 GtkWidget*
 gcal_edit_dialog_new (void)
@@ -570,10 +974,13 @@ gcal_edit_dialog_set_time_format (GcalEditDialog *dialog,
 {
   g_return_if_fail (GCAL_IS_EDIT_DIALOG (dialog));
 
-  dialog->format_24h = use_24h_format;
+  if (dialog->format_24h != use_24h_format)
+  {
+    dialog->format_24h = use_24h_format;
 
-  gcal_time_selector_set_time_format (GCAL_TIME_SELECTOR (dialog->start_time_selector), dialog->format_24h);
-  gcal_time_selector_set_time_format (GCAL_TIME_SELECTOR (dialog->end_time_selector), dialog->format_24h);
+    gcal_time_selector_set_time_format (GCAL_TIME_SELECTOR (dialog->start_time_selector), dialog->format_24h);
+    gcal_time_selector_set_time_format (GCAL_TIME_SELECTOR (dialog->end_time_selector), dialog->format_24h);
+  }
 }
 
 void
@@ -583,10 +990,6 @@ gcal_edit_dialog_set_event_is_new (GcalEditDialog *dialog,
   dialog->event_is_new = event_is_new;
 
   gtk_widget_set_visible (dialog->delete_button, !event_is_new);
-
-  /* FIXME: implement moving events to other sources */
-  gtk_widget_set_sensitive (dialog->sources_button, event_is_new);
-  gtk_button_set_relief (GTK_BUTTON (dialog->sources_button), event_is_new ? GTK_RELIEF_NORMAL : GTK_RELIEF_NONE);
 }
 
 GcalEvent*
@@ -631,8 +1034,6 @@ gcal_edit_dialog_set_event (GcalEditDialog *dialog,
       fill_sources_menu (dialog);
 
       /* Load new event data */
-      gcal_edit_dialog_set_writable (dialog, gcal_manager_is_client_writable (dialog->manager, source));
-
       /* summary */
       summary = gcal_event_get_summary (event);
 
@@ -642,23 +1043,18 @@ gcal_edit_dialog_set_event (GcalEditDialog *dialog,
         gtk_entry_set_text (GTK_ENTRY (dialog->summary_entry), summary);
 
       /* dialog titlebar's title & subtitle */
-      pix = gcal_get_pixbuf_from_color (gcal_event_get_color (event), 16);
+      pix = get_circle_pixbuf_from_color (gcal_event_get_color (event), 16);
       gtk_image_set_from_pixbuf (GTK_IMAGE (dialog->source_image), pix);
       g_object_unref (pix);
 
-      gtk_header_bar_set_subtitle (GTK_HEADER_BAR (dialog->titlebar),
-                                   e_source_get_display_name (source));
+      gtk_label_set_label (GTK_LABEL (dialog->subtitle_label), e_source_get_display_name (source));
 
       /* retrieve start and end dates */
       date_start = gcal_event_get_date_start (event);
       date_start = all_day ? g_date_time_ref (date_start) : g_date_time_to_local (date_start);
 
       date_end = gcal_event_get_date_end (event);
-      /*
-       * This is subtracting what has been added in gcal_edit_dialog_action_button_clicked ().
-       * See bug 769300.
-       */
-      date_end = all_day ? g_date_time_add_days (date_end, -1) : g_date_time_to_local (date_end);
+      date_end = all_day ? g_date_time_ref (date_end) : g_date_time_to_local (date_end);
 
       /* date */
       gcal_date_selector_set_date (GCAL_DATE_SELECTOR (dialog->start_date_selector), date_start);
@@ -669,9 +1065,9 @@ gcal_edit_dialog_set_event (GcalEditDialog *dialog,
       gcal_time_selector_set_time (GCAL_TIME_SELECTOR (dialog->end_time_selector), date_end);
 
       /* all_day  */
-      gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (dialog->all_day_check), all_day);
       gtk_widget_set_sensitive (dialog->start_time_selector, dialog->writable && !all_day);
       gtk_widget_set_sensitive (dialog->end_time_selector, dialog->writable && !all_day);
+      gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (dialog->all_day_check), all_day);
 
       /* location */
       gtk_entry_set_text (GTK_ENTRY (dialog->location_entry), gcal_event_get_location (event));
@@ -681,9 +1077,14 @@ gcal_edit_dialog_set_event (GcalEditDialog *dialog,
                                 gcal_event_get_description (event),
                                 -1);
 
+      gcal_edit_dialog_set_writable (dialog, gcal_manager_is_client_writable (dialog->manager, source));
+
       g_clear_pointer (&date_start, g_date_time_unref);
       g_clear_pointer (&date_end, g_date_time_unref);
 
+      /* Setup the alarms */
+      setup_alarms (dialog);
+
 out:
       g_object_notify (G_OBJECT (dialog), "event");
 
