@@ -41,7 +41,7 @@ typedef struct
 
   GdkWindow      *event_window;
 
-  /**
+  /*
    * the cell on which its drawn the first day of the month, in the first row, 0 for the first cell, 1 for the second,
    * and so on, this takes first_weekday into account already.
    */
@@ -52,18 +52,17 @@ typedef struct
    */
   gint            keyboard_cell;
 
-  /**
+  /*
    * first day of the week according to user locale, being
    * 0 for Sunday, 1 for Monday and so on */
   gint            first_weekday;
 
-  /**
-   * button_down/up flag
-   * These keeps the cell index, meaning 0 for left top, 1 for the next in the row from left to right, etc.
-   * Note that this does not take into account the disabled row before the first active row.
+  /*
+   * The start & end dates of the selection. We use datetimes to allow the user to navigate between months using
+   * the keyboard.
    */
-  gint            start_mark_cell;
-  gint            end_mark_cell;
+  GDateTime      *start_mark_cell;
+  GDateTime      *end_mark_cell;
 
   /**
    * This flags signals when the cursor is over an overflow indicator and when once has been clicked
@@ -81,8 +80,12 @@ typedef struct
 
   icaltimetype   *current_date;
 
+  /* The cell hovered during Drag and Drop */
+  gint            dnd_cell;
+
   /* property */
   icaltimetype   *date;
+  GcalManager    *manager;
 } GcalMonthViewPrivate;
 
 enum
@@ -177,6 +180,8 @@ static GList*         gcal_month_view_get_children_by_uuid  (GcalView       *vie
 
 static void           gcal_view_interface_init              (GcalViewInterface *iface);
 
+static void           cancel_selection                      (GcalMonthView *month_view);
+
 G_DEFINE_TYPE_WITH_CODE (GcalMonthView, gcal_month_view,GCAL_TYPE_SUBSCRIBER_VIEW,
                          G_ADD_PRIVATE (GcalMonthView)
                          G_IMPLEMENT_INTERFACE (GCAL_TYPE_VIEW, gcal_view_interface_init));
@@ -191,6 +196,19 @@ real_cell (gint     cell,
   return rtl ? MIRROR (cell, 0, 7) - 1 : cell;
 }
 
+static void
+cancel_selection (GcalMonthView *month_view)
+{
+  GtkWidget *widget;
+  GcalMonthViewPrivate *priv;
+
+  widget = GTK_WIDGET (month_view);
+  priv = gcal_month_view_get_instance_private (GCAL_MONTH_VIEW (widget));
+
+  g_clear_pointer (&priv->start_mark_cell, g_date_time_unref);
+  g_clear_pointer (&priv->end_mark_cell, g_date_time_unref);
+}
+
 static void
 get_cell_position (GcalMonthView *view,
                    gint           cell,
@@ -235,66 +253,65 @@ show_popover_for_position (GcalMonthView *view,
   GcalSubscriberViewPrivate *ppriv;
   GcalMonthViewPrivate *priv;
   GtkWidget *widget;
-
-  gint start_day, end_day;
+  GDateTime *end_dt,*start_dt;
 
   widget = GTK_WIDGET (view);
   priv = gcal_month_view_get_instance_private (GCAL_MONTH_VIEW (widget));
   ppriv = GCAL_SUBSCRIBER_VIEW (widget)->priv;
+  start_dt = priv->start_mark_cell;
+  end_dt = priv->end_mark_cell;
 
-  if (priv->start_mark_cell < 0)
+  if (priv->start_mark_cell == NULL)
     return FALSE;
 
-  start_day = priv->start_mark_cell - priv->days_delay + 1;
-  end_day = priv->end_mark_cell - priv->days_delay + 1;
-
-  if (priv->pressed_overflow_indicator != -1 && priv->start_mark_cell == priv->end_mark_cell &&
+  if (priv->pressed_overflow_indicator != -1 &&
+      g_date_time_equal (start_dt, end_dt) &&
       g_hash_table_contains (ppriv->overflow_cells, GINT_TO_POINTER (priv->pressed_overflow_indicator)))
     {
       priv->hovered_overflow_indicator = priv->pressed_overflow_indicator;
 
-      rebuild_popover_for_day (GCAL_MONTH_VIEW (widget), end_day);
+      rebuild_popover_for_day (GCAL_MONTH_VIEW (widget), g_date_time_get_day_of_month (end_dt));
       gtk_widget_show_all (priv->overflow_popover);
 
       gtk_widget_queue_draw (widget);
       priv->pressed_overflow_indicator = -1;
-      priv->start_mark_cell = -1;
-      priv->end_mark_cell = -1;
+
+      cancel_selection (GCAL_MONTH_VIEW (widget));
       return TRUE;
     }
   else
     {
-      GDateTime *start_dt, *end_dt;
-
-      start_dt = NULL;
-      end_dt = NULL;
+      gboolean should_clear_end = FALSE;
 
       /* Swap dates if start > end */
-      if (start_day > end_day)
+      if (g_date_time_compare (start_dt, end_dt) > 0)
         {
-          gint aux;
-          aux = end_day;
-          end_day = start_day;
-          start_day = aux;
+          GDateTime *aux;
+          aux = start_dt;
+          start_dt = end_dt;
+          end_dt = aux;
         }
 
-      start_dt = g_date_time_new_local (priv->date->year, priv->date->month, start_day, 0, 0, 0);
-
       /* Only setup an end date when days are different */
-      if (start_day != end_day)
+      if (!g_date_time_equal (start_dt, end_dt))
         {
           GDateTime *tmp_dt;
 
-          tmp_dt = g_date_time_new_local (priv->date->year, priv->date->month, end_day, 0, 0, 0);
+          tmp_dt = g_date_time_new_local (g_date_time_get_year (end_dt),
+                                          g_date_time_get_month (end_dt),
+                                          g_date_time_get_day_of_month (end_dt),
+                                          0, 0, 0);
           end_dt = g_date_time_add_days (tmp_dt, 1);
 
+          should_clear_end = TRUE;
+
           g_clear_pointer (&tmp_dt, g_date_time_unref);
         }
 
       g_signal_emit_by_name (GCAL_VIEW (widget), "create-event", start_dt, end_dt, x, y);
 
-      g_clear_pointer (&start_dt, g_date_time_unref);
-      g_clear_pointer (&end_dt, g_date_time_unref);
+      if (should_clear_end)
+        g_clear_pointer (&end_dt, g_date_time_unref);
     }
 
   gtk_widget_queue_draw (widget);
@@ -670,7 +687,7 @@ gcal_month_view_key_press (GtkWidget   *widget,
   GcalMonthViewPrivate *priv;
   gboolean selection, valid_key;
   gdouble x, y;
-  gint min, max, diff, month_change;
+  gint min, max, diff, month_change, current_day;
 
   g_return_val_if_fail (GCAL_IS_MONTH_VIEW (widget), FALSE);
 
@@ -679,6 +696,7 @@ gcal_month_view_key_press (GtkWidget   *widget,
   valid_key = FALSE;
   diff = 0;
   month_change = 0;
+  current_day = priv->keyboard_cell - priv->days_delay + 1;
   min = priv->days_delay;
   max = priv->days_delay + icaltime_days_in_month (priv->date->month, priv->date->year) - 1;
 
@@ -687,10 +705,8 @@ gcal_month_view_key_press (GtkWidget   *widget,
    * focused cell as the start, and then update the end mark after updating the
    * focused cell.
    */
-  if (selection && priv->start_mark_cell == -1)
-    {
-      priv->start_mark_cell = priv->keyboard_cell;
-    }
+  if (selection && priv->start_mark_cell == NULL)
+      priv->start_mark_cell = g_date_time_new_local (priv->date->year, priv->date->month, current_day, 0, 0, 0);
 
   switch (event->keyval)
     {
@@ -719,15 +735,15 @@ gcal_month_view_key_press (GtkWidget   *widget,
        * If it's not on the selection mode (i.e. shift is not pressed), we should
        * simulate it by changing the start & end selected cells = keyboard cell.
        */
-      if (!selection && priv->start_mark_cell == -1 && priv->end_mark_cell == -1)
-        priv->start_mark_cell = priv->end_mark_cell = priv->keyboard_cell;
+      if (!selection && priv->start_mark_cell == NULL && priv->end_mark_cell == NULL)
+        priv->start_mark_cell = priv->end_mark_cell = g_date_time_new_local (priv->date->year, priv->date->month, current_day, 0, 0, 0);
 
-      get_cell_position (GCAL_MONTH_VIEW (widget), real_cell (priv->end_mark_cell, priv->k), &x, &y);
+      get_cell_position (GCAL_MONTH_VIEW (widget), real_cell (current_day + priv->days_delay - 1, priv->k), &x, &y);
       show_popover_for_position (GCAL_MONTH_VIEW (widget), x, y, FALSE);
       break;
 
     case GDK_KEY_Escape:
-      priv->start_mark_cell = priv->end_mark_cell = -1;
+      cancel_selection (GCAL_MONTH_VIEW (widget));
       break;
 
     default:
@@ -761,14 +777,16 @@ gcal_month_view_key_press (GtkWidget   *widget,
       g_object_notify (G_OBJECT (widget), "active-date");
     }
 
+  current_day = priv->keyboard_cell - priv->days_delay + 1;
+
   if (selection)
     {
-      priv->end_mark_cell = priv->keyboard_cell;
+      priv->end_mark_cell = g_date_time_new_local (priv->date->year, priv->date->month, current_day, 0, 0, 0);
     }
   else if (!selection && valid_key)
     {
       /* Cancel selection if SHIFT is not pressed */
-      priv->start_mark_cell = priv->end_mark_cell = -1;
+      cancel_selection (GCAL_MONTH_VIEW (widget));
     }
 
   gtk_widget_queue_draw (widget);
@@ -776,6 +794,130 @@ gcal_month_view_key_press (GtkWidget   *widget,
   return TRUE;
 }
 
+/*
+ * Drag and Drop functions
+ */
+static gint
+get_dnd_cell (GtkWidget *widget,
+              gint       x,
+              gint       y)
+{
+  GcalMonthViewPrivate *priv;
+  gint cell;
+
+  priv = gcal_month_view_get_instance_private (GCAL_MONTH_VIEW (widget));
+
+  cell = gather_button_event_data (GCAL_MONTH_VIEW (widget), x, y, NULL, NULL, NULL);
+  cell = real_cell (cell, priv->k);
+
+  if (cell >= priv->days_delay &&
+      cell < g_date_get_days_in_month (priv->date->month, priv->date->year) + priv->days_delay)
+    {
+      return cell;
+    }
+
+  return -1;
+}
+
+static gboolean
+gcal_month_view_drag_motion (GtkWidget      *widget,
+                             GdkDragContext *context,
+                             gint            x,
+                             gint            y,
+                             guint           time)
+{
+  GcalMonthViewPrivate *priv;
+
+  priv = gcal_month_view_get_instance_private (GCAL_MONTH_VIEW (widget));
+  priv->dnd_cell = get_dnd_cell (widget, x, y);
+
+  /* Setup the drag highlight */
+  if (priv->dnd_cell != -1)
+    gtk_drag_highlight (widget);
+  else
+    gtk_drag_unhighlight (widget);
+
+  /*
+   * Sets the status of the drag - if it fails, sets the action to 0 and
+   * aborts the drag with FALSE.
+   */
+  gdk_drag_status (context,
+                   priv->dnd_cell == -1 ? 0 : GDK_ACTION_COPY,
+                   time);
+
+  gtk_widget_queue_draw (widget);
+
+  return priv->dnd_cell != -1;
+}
+
+static gboolean
+gcal_month_view_drag_drop (GtkWidget      *widget,
+                           GdkDragContext *context,
+                           gint            x,
+                           gint            y,
+                           guint           time)
+{
+  GcalMonthViewPrivate *priv = gcal_month_view_get_instance_private (GCAL_MONTH_VIEW (widget));
+  GtkWidget *event_widget;
+  GDateTime *start_dt, *end_dt;
+  GcalEvent *event;
+  gint cell, diff;
+
+  cell = get_dnd_cell (widget, x, y);
+  event_widget = gtk_drag_get_source_widget (context);
+  event = gcal_event_widget_get_event (GCAL_EVENT_WIDGET (event_widget));
+
+  /* Move the event's date */
+  start_dt = gcal_event_get_date_start (event);
+  end_dt = gcal_event_get_date_end (event);
+
+  diff = cell - priv->days_delay - g_date_time_get_day_of_month (start_dt) + 1;
+
+  if (diff != 0)
+    {
+      GDateTime *new_start = g_date_time_add_days (start_dt, diff);
+
+      gcal_event_set_date_start (event, new_start);
+
+      /* The event may have a NULL end date, so we have to check it here */
+      if (end_dt)
+        {
+          GDateTime *new_end = g_date_time_add_days (end_dt, diff);
+
+          gcal_event_set_date_end (event, new_end);
+          g_clear_pointer (&new_end, g_date_time_unref);
+        }
+
+      gcal_manager_update_event (priv->manager, event);
+
+      g_clear_pointer (&new_start, g_date_time_unref);
+    }
+
+  /* Cancel the DnD */
+  priv->dnd_cell = -1;
+  gtk_drag_unhighlight (widget);
+
+  gtk_drag_finish (context, TRUE, FALSE, time);
+
+  gtk_widget_queue_draw (widget);
+
+  return TRUE;
+}
+
+static void
+gcal_month_view_drag_leave (GtkWidget      *widget,
+                            GdkDragContext *context,
+                            guint           time)
+{
+  GcalMonthViewPrivate *priv = gcal_month_view_get_instance_private (GCAL_MONTH_VIEW (widget));
+
+  /* Cancel the drag */
+  priv->dnd_cell = -1;
+  gtk_drag_unhighlight (widget);
+
+  gtk_widget_queue_draw (widget);
+}
+
 static void
 add_new_event_button_cb (GtkWidget *button,
                          gpointer   user_data)
@@ -820,6 +962,9 @@ gcal_month_view_class_init (GcalMonthViewClass *klass)
   widget_class->button_release_event = gcal_month_view_button_release;
   widget_class->direction_changed = gcal_month_view_direction_changed;
   widget_class->key_press_event = gcal_month_view_key_press;
+  widget_class->drag_motion = gcal_month_view_drag_motion;
+  widget_class->drag_drop = gcal_month_view_drag_drop;
+  widget_class->drag_leave = gcal_month_view_drag_leave;
 
   subscriber_view_class = GCAL_SUBSCRIBER_VIEW_CLASS (klass);
   subscriber_view_class->get_child_cell = gcal_month_view_get_child_cell;
@@ -841,8 +986,7 @@ gcal_month_view_init (GcalMonthView *self)
 
   priv = gcal_month_view_get_instance_private (self);
 
-  priv->start_mark_cell = -1;
-  priv->end_mark_cell = -1;
+  cancel_selection (self);
 
   priv->pressed_overflow_indicator = -1;
   priv->hovered_overflow_indicator = -1;
@@ -870,6 +1014,13 @@ gcal_month_view_init (GcalMonthView *self)
   gtk_container_add (GTK_CONTAINER (grid), priv->popover_title);
   gtk_container_add (GTK_CONTAINER (grid), priv->events_list_box);
   gtk_container_add (GTK_CONTAINER (grid), button);
+
+  /* Setup the month view as a drag n' drop destination */
+  gtk_drag_dest_set (GTK_WIDGET (self),
+                     0,
+                     NULL,
+                     0,
+                     GDK_ACTION_MOVE);
 }
 
 static void
@@ -905,8 +1056,8 @@ gcal_month_view_set_property (GObject       *object,
         priv->date = g_value_dup_boxed (value);
         priv->days_delay = (time_day_of_week (1, priv->date->month - 1, priv->date->year) - priv->first_weekday + 7) % 7;
         priv->keyboard_cell = priv->days_delay + (priv->date->day - 1);
-        priv->start_mark_cell = -1;
-        priv->end_mark_cell = -1;
+
+        cancel_selection (GCAL_MONTH_VIEW (object));
 
         ppriv->children_changed = TRUE;
         gtk_widget_queue_resize (GTK_WIDGET (object));
@@ -1420,10 +1571,28 @@ gcal_month_view_draw (GtkWidget *widget,
   sw = 1 - 2 * priv->k;
 
   /* active cells */
-  if (priv->start_mark_cell > -1 && priv->end_mark_cell > -1)
+  if (priv->start_mark_cell && priv->end_mark_cell)
     {
-      lower_mark = priv->start_mark_cell - priv->days_delay + 1;
-      upper_mark = priv->end_mark_cell  - priv->days_delay + 1;
+      gint start_year, end_year, start_month, end_month;
+
+      upper_mark = g_date_time_get_day_of_month(priv->end_mark_cell);
+
+      start_year = g_date_time_get_year (priv->start_mark_cell);
+      end_year = g_date_time_get_year (priv->end_mark_cell);
+      start_month = g_date_time_get_month (priv->start_mark_cell);
+      end_month = g_date_time_get_month (priv->end_mark_cell);
+
+      if (start_year == end_year && start_month == end_month)
+        {
+          lower_mark = g_date_time_get_day_of_month(priv->start_mark_cell);
+        }
+      else
+        {
+          if (start_year != end_year)
+            lower_mark = start_year < end_year ? 1 : icaltime_days_in_month (end_month, end_year);
+          else
+            lower_mark = start_month < end_month ? 1 : icaltime_days_in_month (end_month, end_year);
+        }
 
       if (lower_mark > upper_mark)
         {
@@ -1497,6 +1666,17 @@ gcal_month_view_draw (GtkWidget *widget,
   pango_font_description_free (font_desc);
   gtk_style_context_restore (context);
 
+  /* Drag and Drop highlight */
+  if (priv->dnd_cell != -1)
+    {
+      gtk_render_background (context,
+                             cr,
+                             floor (cell_width * real_cell (priv->dnd_cell % 7, priv->k)),
+                             floor (cell_height * (priv->dnd_cell / 7 + first_row_gap) + start_grid_y),
+                             cell_width + 1,
+                             cell_height + 1);
+    }
+
   /* grid numbers */
   gtk_style_context_get (context, state, "font", &font_desc, NULL);
   pango_layout_set_font_description (layout, font_desc);
@@ -1657,13 +1837,36 @@ gcal_month_view_draw (GtkWidget *widget,
   gtk_style_context_restore (context);
 
   /* selection borders */
-  if (priv->start_mark_cell != -1 && priv->end_mark_cell != -1)
+  if (priv->start_mark_cell && priv->end_mark_cell)
    {
      gint first_mark, last_mark;
      gint pos_x2, pos_y2;
+     gint start_year, end_year, start_month, end_month;
+
+     start_year = g_date_time_get_year (priv->start_mark_cell);
+     end_year = g_date_time_get_year (priv->end_mark_cell);
+     start_month = g_date_time_get_month (priv->start_mark_cell);
+     end_month = g_date_time_get_month (priv->end_mark_cell);
+
+     last_mark = real_cell (g_date_time_get_day_of_month (priv->end_mark_cell) + priv->days_delay - 1, priv->k);
+
+     if (start_year == end_year && start_month == end_month)
+      {
+        first_mark = real_cell (g_date_time_get_day_of_month (priv->start_mark_cell) + priv->days_delay - 1, priv->k);
+      }
+     else
+      {
+        gint start_cell, end_cell;
+
+        start_cell = priv->days_delay;
+        end_cell = icaltime_days_in_month (end_month, end_year) + priv->days_delay - 1;
+
+        if (start_year != end_year)
+          first_mark = start_year < end_year ? real_cell (start_cell, priv->k) : real_cell(end_cell, priv->k);
+        else
+          first_mark = start_month < end_month ? real_cell (start_cell, priv->k) : real_cell (end_cell, priv->k);
+      }
 
-     first_mark = real_cell (priv->start_mark_cell, priv->k);
-     last_mark = real_cell (priv->end_mark_cell, priv->k);
      if (first_mark > last_mark)
        {
          gint tmp = first_mark;
@@ -1820,8 +2023,13 @@ gcal_month_view_button_press (GtkWidget      *widget,
 
   if (clicked_cell >= priv->days_delay && clicked_cell < days)
     {
-      priv->start_mark_cell = clicked_cell;
+      g_clear_pointer (&priv->start_mark_cell, g_date_time_unref);
+
       priv->keyboard_cell = clicked_cell;
+      priv->start_mark_cell = g_date_time_new_local (priv->date->year, priv->date->month,
+                                                     priv->keyboard_cell - priv->days_delay + 1,
+                                                     0, 0, 0);
+      gtk_widget_queue_draw (widget);
     }
 
   if (pressed_indicator && g_hash_table_contains (ppriv->overflow_cells, GINT_TO_POINTER (clicked_cell)))
@@ -1857,21 +2065,46 @@ gcal_month_view_motion_notify_event (GtkWidget      *widget,
   new_end_cell = gather_button_event_data (GCAL_MONTH_VIEW (widget), event->x, event->y, &hovered_indicator, NULL, NULL);
   new_end_cell = real_cell (new_end_cell, priv->k);
 
-  if (priv->start_mark_cell != -1)
+  if (priv->start_mark_cell)
     {
       if (!(event->state & GDK_BUTTON_PRESS_MASK))
         return TRUE;
 
       if (new_end_cell >= priv->days_delay && new_end_cell < days)
         {
+          guint previous_end_cell;
+
+          previous_end_cell = new_end_cell;
+
+          /* Let the keyboard focus track the pointer */
           priv->keyboard_cell = new_end_cell;
 
-          if (priv->end_mark_cell != priv->start_mark_cell)
-            priv->hovered_overflow_indicator = -1;
-          if (priv->end_mark_cell != new_end_cell)
+          /*
+           * When there is a previously set end mark, it should be
+           * cleared before assigning a new one.
+           */
+          if (priv->end_mark_cell)
+            {
+              previous_end_cell = g_date_time_get_day_of_month (priv->end_mark_cell) - 1;
+
+              g_clear_pointer (&priv->end_mark_cell, g_date_time_unref);
+            }
+
+          if (priv->start_mark_cell &&
+              priv->end_mark_cell &&
+              !g_date_time_equal (priv->start_mark_cell, priv->end_mark_cell))
+            {
+              priv->hovered_overflow_indicator = -1;
+            }
+
+          if (previous_end_cell != new_end_cell)
             gtk_widget_queue_draw (widget);
 
-          priv->end_mark_cell = new_end_cell;
+          priv->end_mark_cell = g_date_time_new_local (priv->date->year,
+                                                       priv->date->month,
+                                                       new_end_cell - priv->days_delay + 1,
+                                                       0, 0, 0);
+
           return TRUE;
         }
     }
@@ -1902,24 +2135,28 @@ gcal_month_view_button_release (GtkWidget      *widget,
 {
   GcalMonthViewPrivate *priv;
   gdouble x, y;
-  gint days;
+  gint days, current_day;
   gboolean released_indicator = FALSE;
 
   priv = gcal_month_view_get_instance_private (GCAL_MONTH_VIEW (widget));
   days = priv->days_delay + icaltime_days_in_month (priv->date->month, priv->date->year);
 
-  priv->end_mark_cell = gather_button_event_data (GCAL_MONTH_VIEW (widget), event->x, event->y, &released_indicator, &x, &y);
-  priv->end_mark_cell = real_cell (priv->end_mark_cell, priv->k);
+  current_day = gather_button_event_data (GCAL_MONTH_VIEW (widget), event->x, event->y, &released_indicator, &x, &y);
+  current_day = real_cell (current_day, priv->k);
 
-  if (priv->end_mark_cell >= priv->days_delay && priv->end_mark_cell < days)
+  if (current_day >= priv->days_delay && current_day < days)
     {
-      priv->keyboard_cell = priv->end_mark_cell;
+      g_clear_pointer (&priv->end_mark_cell, g_date_time_unref);
+
+      priv->keyboard_cell = current_day;
+      priv->end_mark_cell = g_date_time_new_local (priv->date->year, priv->date->month, current_day - priv->days_delay + 1, 0, 0, 0);
       return show_popover_for_position (GCAL_MONTH_VIEW (widget), x, y, released_indicator);
     }
   else
     {
       /* If the button is released over an invalid cell, entirely cancel the selection */
-      priv->start_mark_cell = priv->end_mark_cell = -1;
+      cancel_selection (GCAL_MONTH_VIEW (widget));
+
       gtk_widget_queue_draw (widget);
 
       return FALSE;
@@ -1973,8 +2210,8 @@ gcal_month_view_clear_state (GcalSubscriberView *subscriber)
 
   priv = gcal_month_view_get_instance_private (GCAL_MONTH_VIEW (subscriber));
 
-  priv->start_mark_cell = -1;
-  priv->end_mark_cell = -1;
+  cancel_selection (GCAL_MONTH_VIEW (subscriber));
+
   priv->pressed_overflow_indicator = -1;
 
   gtk_widget_hide (priv->overflow_popover);
@@ -2038,12 +2275,8 @@ gcal_month_view_get_final_date (GcalView *view)
 static void
 gcal_month_view_clear_marks (GcalView *view)
 {
-  GcalMonthViewPrivate *priv;
+  cancel_selection (GCAL_MONTH_VIEW (view));
 
-  priv = gcal_month_view_get_instance_private (GCAL_MONTH_VIEW (view));
-
-  priv->start_mark_cell = -1;
-  priv->end_mark_cell = -1;
   gtk_widget_queue_draw (GTK_WIDGET (view));
 }
 
@@ -2108,3 +2341,12 @@ gcal_month_view_set_use_24h_format (GcalMonthView *view,
   priv = gcal_month_view_get_instance_private (view);
   priv->use_24h_format = use_24h;
 }
+
+void
+gcal_month_view_set_manager (GcalMonthView *view,
+                             GcalManager   *manager)
+{
+  GcalMonthViewPrivate *priv = gcal_month_view_get_instance_private (view);
+
+  priv->manager = manager;
+}
