@@ -113,6 +113,7 @@ struct _GcalWindow
   GtkWidget           *source_dialog;
   gint                 refresh_timeout;
   gint                 refresh_timeout_id;
+  gint                 open_edit_dialog_timeout_id;
 
   /* temp to keep event_creation */
   gboolean             open_edit_dialog;
@@ -827,7 +828,7 @@ static GtkWidget*
 make_row_from_source (GcalWindow *window,
                       ESource    *source)
 {
-  GtkWidget *label, *icon, *separator, *checkbox, *box, *row;
+  GtkWidget *label, *icon, *checkbox, *box, *row;
   GtkStyleContext *context;
   GdkPixbuf *pixbuf;
   GdkRGBA color;
@@ -846,17 +847,16 @@ make_row_from_source (GcalWindow *window,
 
   /* source color icon */
   get_color_name_from_source (source, &color);
-  pixbuf = gcal_get_pixbuf_from_color (&color, 16);
+  pixbuf = get_circle_pixbuf_from_color (&color, 16);
   icon = gtk_image_new_from_pixbuf (pixbuf);
 
+  gtk_style_context_add_class (gtk_widget_get_style_context (icon), "calendar-color-image");
+
   /* source name label */
   label = gtk_label_new (e_source_get_display_name (source));
   gtk_label_set_xalign (GTK_LABEL (label), 0.0);
   gtk_widget_set_hexpand (label, TRUE);
 
-  /* vertical separator */
-  separator = gtk_separator_new (GTK_ORIENTATION_VERTICAL);
-
   /* checkbox */
   checkbox = gtk_check_button_new ();
   gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (checkbox), gcal_manager_source_enabled (window->manager, source));
@@ -864,7 +864,6 @@ make_row_from_source (GcalWindow *window,
 
   gtk_container_add (GTK_CONTAINER (box), icon);
   gtk_container_add (GTK_CONTAINER (box), label);
-  gtk_container_add (GTK_CONTAINER (box), separator);
   gtk_container_add (GTK_CONTAINER (box), checkbox);
   gtk_container_add (GTK_CONTAINER (row), box);
 
@@ -907,21 +906,12 @@ source_row_activated (GtkListBox    *listbox,
                       GtkListBoxRow *row,
                       gpointer       user_data)
 {
-  GcalWindow *window;
-  ESource *source;
+  GtkWidget *check;
 
-  window = GCAL_WINDOW (user_data);
-  source = g_object_get_data (G_OBJECT (row), "source");
+  check = g_object_get_data (G_OBJECT (row), "check");
 
-  if (source == NULL)
-    return;
-
-  gcal_source_dialog_set_source (GCAL_SOURCE_DIALOG (window->source_dialog), source);
-  gcal_source_dialog_set_mode (GCAL_SOURCE_DIALOG (window->source_dialog), GCAL_SOURCE_DIALOG_MODE_EDIT);
-
-  gtk_widget_hide (window->calendar_popover);
-
-  gtk_widget_show (window->source_dialog);
+  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (check),
+                                !gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (check)));
 }
 
 static void
@@ -1232,16 +1222,17 @@ schedule_open_edit_dialog_by_uuid (OpenEditDialogData *edit_dialog_data)
   widgets = gcal_view_get_children_by_uuid (GCAL_VIEW (window->month_view), edit_dialog_data->uuid);
   if (widgets != NULL)
     {
+      window->open_edit_dialog_timeout_id = 0;
+
       event_activated (NULL, widgets->data, edit_dialog_data->window);
       g_list_free (widgets);
       g_free (edit_dialog_data->uuid);
       g_free (edit_dialog_data);
-      return FALSE;
-    }
-  else
-    {
-      return TRUE;
+
+      return G_SOURCE_REMOVE;
     }
+
+  return G_SOURCE_CONTINUE;
 }
 
 static void
@@ -1460,6 +1451,12 @@ gcal_window_finalize (GObject *object)
       window->save_geometry_timeout_id = 0;
     }
 
+  if (window->open_edit_dialog_timeout_id > 0)
+    {
+      g_source_remove (window->open_edit_dialog_timeout_id);
+      window->open_edit_dialog_timeout_id = 0;
+    }
+
   /* If we have a queued event to delete, remove it now */
   if (window->event_to_delete)
     {
@@ -1703,6 +1700,8 @@ gcal_window_open_event_by_uuid (GcalWindow  *window,
       OpenEditDialogData *edit_dialog_data = g_new0 (OpenEditDialogData, 1);
       edit_dialog_data->window = window;
       edit_dialog_data->uuid = g_strdup (uuid);
-      g_timeout_add_seconds (2, (GSourceFunc) schedule_open_edit_dialog_by_uuid, edit_dialog_data);
+      window->open_edit_dialog_timeout_id = g_timeout_add_seconds (2,
+                                                                   (GSourceFunc) schedule_open_edit_dialog_by_uuid,
+                                                                   edit_dialog_data);
     }
 }
