@@ -93,7 +93,7 @@ struct _GcalYearView
   gboolean      use_24h_format;
 
   /* show week numbers from GNOME Shell settings */
-  GSettings    *shell_settings;
+  GSettings    *calendar_settings;
   gboolean      show_week_numbers;
 
   /* text direction factors */
@@ -527,6 +527,40 @@ update_sidebar_headers (GtkListBoxRow *row,
   g_clear_pointer (&row_date, g_date_time_unref);
 }
 
+static void
+calculate_coord_for_date (GcalYearView *year_view,
+                          gint          day,
+                          gint          month,
+                          gboolean      is_title,
+                          GdkRectangle *rect)
+{
+  gint cell_x, cell_y, clicked_cell, sw;
+
+  if (is_title ||
+      year_view->selected_data->start_day != year_view->selected_data->end_day ||
+      year_view->selected_data->start_month != year_view->selected_data->end_month)
+    {
+      rect->x = year_view->navigator_grid->coordinates[month].x;
+      rect->y = year_view->navigator_grid->coordinates[month].y;
+      rect->width = year_view->column_width;
+      rect->height =  year_view->row_height;
+
+      return;
+    }
+
+  /* else */
+  sw = 1 - 2 * year_view->k;
+  clicked_cell = day + ((time_day_of_week (1, month, year_view->date->year) - year_view->first_weekday + 7) % 7) - 1;
+
+  cell_x = (clicked_cell % 7 + year_view->k + year_view->show_week_numbers) * year_view->navigator_grid->box_side * sw;
+  cell_x += (year_view->k * year_view->navigator_grid->box_side * (7 + year_view->show_week_numbers));
+  cell_y = ((clicked_cell / 7 + 1) * year_view->navigator_grid->box_side);
+
+  rect->x = cell_x + year_view->navigator_grid->coordinates[month].x;
+  rect->y = cell_y + year_view->navigator_grid->coordinates[month].y;
+  rect->width = rect->height =  year_view->navigator_grid->box_side;
+}
+
 static gint
 sidebar_sort_func (GtkListBoxRow *row1,
                    GtkListBoxRow *row2,
@@ -1071,10 +1105,7 @@ navigator_button_release_cb (GcalYearView   *year_view,
       box = gtk_bin_get_child (GTK_BIN (year_view->popover));
       gtk_widget_set_size_request (box, 200, year_view->navigator_grid->box_side * 2 * 7);
 
-      /* FIXME: improve rect */
-      rect.x = event->x;
-      rect.y = event->y;
-      rect.width = rect.height = 1;
+      calculate_coord_for_date (year_view, day, month, is_title, &rect);
       gtk_popover_set_pointing_to (GTK_POPOVER (year_view->popover), &rect);
 
       /* FIXME: do no show over selected days */
@@ -1401,7 +1432,7 @@ gcal_year_view_finalize (GObject *object)
 
   g_clear_pointer (&year_view->date, g_free);
 
-  g_clear_object (&year_view->shell_settings);
+  g_clear_object (&year_view->calendar_settings);
 
   G_OBJECT_CLASS (gcal_year_view_parent_class)->finalize (object);
 }
@@ -1693,7 +1724,7 @@ gcal_year_view_component_removed (ECalDataModelSubscriber *subscriber,
   GList *children, *l;
   ESource *source;
   gchar *uuid;
-  gboolean update_sidebar_needed = FALSE;
+  gint number_of_children;
 
   source = e_client_get_source (E_CLIENT (client));
   if (rid != NULL)
@@ -1702,6 +1733,8 @@ gcal_year_view_component_removed (ECalDataModelSubscriber *subscriber,
     uuid = g_strdup_printf ("%s:%s", e_source_get_uid (source), uid);
 
   children = gtk_container_get_children (GTK_CONTAINER (year_view->events_sidebar));
+  number_of_children = g_list_length (children);
+
   for (l = children; l != NULL; l = g_list_next (l))
     {
       GcalEventWidget *child_widget;
@@ -1710,22 +1743,25 @@ gcal_year_view_component_removed (ECalDataModelSubscriber *subscriber,
       child_widget = GCAL_EVENT_WIDGET (gtk_bin_get_child (GTK_BIN (l->data)));
       event = gcal_event_widget_get_event (child_widget);
 
-
-      if (child_widget != NULL && g_strcmp0 (uuid, gcal_event_get_uid (event)) == 0)
+      if (g_strcmp0 (uuid, gcal_event_get_uid (event)) == 0)
         {
-          if (g_list_length (children) == 1)
-            update_sidebar_needed = TRUE;
           gtk_widget_destroy (GTK_WIDGET (l->data));
+          number_of_children--;
         }
     }
-  g_list_free (children);
-  g_free (uuid);
 
-  if (update_sidebar_needed)
+  /*
+   * No children left visible, all the events were removed and now we have to show the
+   * 'No Events' placeholder.
+   */
+  if (number_of_children == 0)
     {
       update_no_events_page (GCAL_YEAR_VIEW (subscriber));
       gtk_stack_set_visible_child_name (GTK_STACK (year_view->navigator_stack), "no-events");
     }
+
+  g_list_free (children);
+  g_free (uuid);
 }
 
 static void
@@ -1811,9 +1847,9 @@ gcal_year_view_init (GcalYearView *self)
   self->end_selected_date->zone = e_cal_util_get_system_timezone ();
 
   /* bind GNOME Shell' show week numbers property to GNOME Calendar's one */
-  self->shell_settings = g_settings_new ("org.gnome.shell.calendar");
-  g_settings_bind (self->shell_settings, "show-weekdate", self, "show-week-numbers", G_SETTINGS_BIND_DEFAULT);
-  g_signal_connect_swapped (self->shell_settings, "changed::show-weekdate", G_CALLBACK (gtk_widget_queue_draw), self);
+  self->calendar_settings = g_settings_new ("org.gnome.desktop.calendar");
+  g_settings_bind (self->calendar_settings, "show-weekdate", self, "show-week-numbers", G_SETTINGS_BIND_DEFAULT);
+  g_signal_connect_swapped (self->calendar_settings, "changed::show-weekdate", G_CALLBACK (gtk_widget_queue_draw), self);
 
   /* layout */
   self->number_of_columns = 4;
